/*
Resettable electronic circuit breaker for motorized hot tub cover
MagicManu 06/2020
https://www.magicmanu.com
*/

#define Current_sensor A0  // The ACS712 sensor analog input pin (-20A 0 +20A => 0 512 1024)
#define Potentiometer A2   // Potentiometer (0 to 1024)
#define ResetButton 12     // RaZ button
#define LedGreen 4         // Green LED ready
#define LedRed 6           // Red LED error
#define Relay 2            // Relay (HIGH = stop)

long TimeRef = 0;   // Time at the start of current is exceeds
int Cpt = 0;        // Number of millisecond of current is exceeds

boolean ButtonState;
float CurrentValue;
float PotenValue;
float RealCurrentValue;

boolean Exceeds = LOW;
boolean STOP = LOW;


void setup() {
  //Serial.begin(9600);
  
  pinMode(Current_sensor, INPUT);
  pinMode(Potentiometer, INPUT);
  pinMode(ResetButton, INPUT_PULLUP);
  pinMode(LedGreen, OUTPUT);
  pinMode(LedRed, OUTPUT);
  pinMode(Relay, OUTPUT);

  digitalWrite(LedGreen, HIGH);
  digitalWrite(LedRed, LOW);
  digitalWrite(Relay, LOW);
}

void loop() {

  ButtonState  = digitalRead(ResetButton);
  CurrentValue = analogRead(Current_sensor);
  PotenValue   = analogRead(Potentiometer);

  // To convert 512-612 to 0-1024 (0A - 6A)
  RealCurrentValue = (CurrentValue - 512) * 10.24;

  // Current is exceeds
  if (RealCurrentValue > PotenValue && !Exceeds && !STOP){
    TimeRef = millis();
    Exceeds = HIGH;
  }

  if (RealCurrentValue < PotenValue){
    Exceeds = LOW;
  }


  Cpt = millis() - TimeRef;

  // 0,25 seconds : stop
  if (Exceeds && Cpt > 250){
    STOP = HIGH;
    digitalWrite(Relay, HIGH);
    digitalWrite(LedGreen, LOW);  
    digitalWrite(LedRed, HIGH);        
  }

  // 5 seconds : we can start
  if (STOP && Cpt > 5000){
    digitalWrite(LedGreen, HIGH); // Ready
    
    // Reset
    if (!ButtonState){
      STOP = LOW;
      digitalWrite(Relay, LOW);
      digitalWrite(LedRed, LOW);  
    }    
  }

  
  //if (Exceeds && RealCurrentValue > 500){
  //  Serial.println((String)"Courant:"+RealCurrentValue+" / "+PotenValue+" CPT="+Cpt);
  //  delay(100);                     
  //}
}
